"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const rimraf_1 = tslib_1.__importDefault(require("rimraf"));
const standards_1 = require("./standards");
function unlink(path, options, callback) {
    // Parse if callback is provided
    if (callback === undefined && typeof options === 'function') {
        callback = options;
        options = { standard: 'secure' };
    }
    // Define standard if none is provided
    if (typeof options.standard === undefined)
        options.standard = 'secure';
    if (callback)
        unlinkCallback(path, options, (err, path) => callback(err, path));
    else
        return unlinkPromise(path, options);
}
exports.unlink = unlink;
// (module).exports = secureRm
// Callback version
function unlinkCallback(path, options, callback) {
    if (options.customStandard) {
        rimraf_1.default(path, {
            unlink: options.customStandard,
            maxBusyTries: options.maxBusyTries || 3,
            disableGlob: options.disableGlob || false
        }, (err) => callback(err, path));
    }
    else if (standards_1.validIDs.includes(options.standard)) {
        rimraf_1.default(path, {
            unlink: standards_1.standards[options.standard].method,
            maxBusyTries: options.maxBusyTries || 3,
            disableGlob: options.disableGlob || false
        }, (err) => callback(err, path));
    }
    else {
        callback(new Error(`'${options.standard}' is not a valid ID. \nList of valid IDs: ${standards_1.validIDs}`), path);
    }
}
// Promise version
function unlinkPromise(path, options) {
    return new Promise((resolve, reject) => {
        if (options.customStandard) {
            rimraf_1.default(path, {
                unlink: options.customStandard,
                maxBusyTries: options.maxBusyTries || 3,
                disableGlob: options.disableGlob || false
            }, (err) => {
                if (err)
                    reject(err);
                else
                    resolve(path);
            });
        }
        else if (standards_1.validIDs.includes(options.standard)) {
            rimraf_1.default(path, {
                unlink: standards_1.standards[options.standard].method,
                maxBusyTries: options.maxBusyTries || 3,
                disableGlob: options.disableGlob || false
            }, (err) => {
                if (err)
                    reject(err);
                else
                    resolve(path);
            });
        }
        else {
            reject(new Error(`'${options.standard}' is not a valid ID. \nList of valid IDs: ${standards_1.validIDs}`));
        }
    });
}
